/*
 *
 * $Id: plain-file.c,v 1.1.2.9 2000/06/13 13:07:20 moritz Exp $
 *
 * old "plain file" backend to fetchipac
 * Copyright (C) 1997 - 2000 Moritz Both
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author can be reached via email: moritz@daneben.de, or by
 * snail mail: Moritz Both, Im Moore 26, 30167 Hannover,
 *             Germany. Phone: +49-511-1610129
 *
 */

#include "ipac.h"
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/param.h>
#include <unistd.h>

/* plain file ipac interface entries */
int files_ipac_open(int flag);
int files_ipac_get_raw_list(char *ag_name, char *login, 
		raw_rule_type **data);
int files_ipac_delete_record(timestamp_t timestamp);
float files_ipac_get_cash(char *login);
int files_ipac_set_cash(char *login, float cash, timestamp_t timest);
float files_ipac_get_price(char *rule_name);
int files_ipac_get_pay_type(char *rule_name);
timestamp_t files_ipac_get_last_paid(char *service_name);
int files_ipac_set_last_paid(char *service_name, float cash);
int files_ipac_close();

static const access_agent_t interface_entry = {
	"files",
	files_ipac_open,
	files_ipac_get_raw_list,
	files_ipac_get_cash,
	files_ipac_set_cash,
	files_ipac_get_price,
	files_ipac_get_pay_type,
	files_ipac_get_last_paid,
	files_ipac_close
};

const access_agent_t *ipac_ac_interface_files() {
	return &interface_entry;
};


int files_ipac_open(int flag)
{
	/* nothing to do. */
	return 0;
}

/*
 * new functions are still not fully supported in plain-file method
 * this function is here for compatibility only - it does not support
 * storage agents and login names. Eats old styled config file
 */
int files_ipac_get_raw_list(char *ag_name, char *login, raw_rule_type **data)
{
	FILE *conf;
	char buf[MAX_RULE_NAME_LENGTH + 100];
	char *tmp, *src, *dst, *buf1, *tmp1;
	raw_rule_type *r1, *r, *r2, *r3;
	int i, n;
	
	conf = fopen(CONFFILE, "r");
	if (!conf) {
		fprintf(stderr, "%s: opening config \"%s\": %s\n",
			me, CONFFILE, sys_errlist[errno]);
		return 1;
	}
	r1 = NULL;
	while(fgets(buf, MAX_RULE_NAME_LENGTH + 100, conf)) {
		if ((*buf=='#')||(*buf=='\n')||(*buf=='\t')||(*buf==' '))
			continue;
		r = new_raw_rule();
		if (r1 == NULL)
			*data = r;
		else
			r1->next = r;
		r1 = r;	r2 = r; r3 = r;
		buf1=xstrdup(buf);
		
		tmp = strsep(&buf1, "|");	// get rule name
		strncpy(r->name, tmp, MAX_RULE_NAME_LENGTH);
		tmp = strsep(&buf1, "|");	// get destination
		strncpy(r->dest, tmp, 5);
		tmp = strsep(&buf1, "|");	// get iface
		strncpy(r->iface, tmp, 9);
		tmp = strsep(&buf1, "|");	// get protocol
		strncpy(r->protocol, tmp, 5);
		src = strsep(&buf1, "|");	// get source address/ports
		dst = strsep(&buf1, "|");

		if (strlen(src)) {
			tmp = strtok(src, " ");		// split source to address/proto
			strncpy(r->snet, tmp, 19);

			tmp = strtok(NULL, " ");	// are there any ports?
			if (tmp)
				strncpy(r->sport, tmp, 19);
			tmp = strtok(NULL, " ");
			while (tmp) { // uh! there are another ports
				r = new_raw_rule();
				r1->next = r;
				strncpy(r->name, r1->name, MAX_RULE_NAME_LENGTH);
				strncpy(r->dest, r1->dest, 5);
				strncpy(r->iface, r1->iface, 9);
				strncpy(r->protocol, r1->protocol, 5);
				strncpy(r->snet, r1->snet, 19);
				strncpy(r->sport, tmp, 19);
				r1 = r;
				tmp = strtok(NULL, " ");
			}
		} else {
			strcpy(r->snet, "0/0");
			strcpy(r->sport, "");
		}

		if (strlen(dst)>1) {
			tmp = strtok(dst, " \n");
			tmp1 = strtok(NULL, " \n");
			r=r2; n=0;
			while (r) {
				strncpy(r->dnet, tmp, 19);
				strncpy(r->dport, tmp1, 19);
				r=r->next;
				n++;
			}
			tmp = strtok(NULL, " \n");
			while (tmp) { // uh-uh!!! whheee.. ugly?
				r2 = r3;
				for (i=0;i<n;i++) {
					r = new_raw_rule();
					r1->next = r;
					strncpy(r->name, r2->name, MAX_RULE_NAME_LENGTH);
					strncpy(r->dest, r2->dest, 5);
					strncpy(r->iface, r2->iface, 9);
					strncpy(r->protocol, r2->protocol, 5);
					strncpy(r->snet, r2->snet, 19);
					strncpy(r->sport, r2->sport, 19);
					strncpy(r->dnet, r2->dnet, 19);
					strncpy(r->dport, tmp, 19);
					r1 = r;
					r2 = r2->next;
				}
				tmp = strtok(NULL, " \n");
			}
			
		} else {
			strcpy(r->dnet, "0/0");
			strcpy(r->dport, "");
		}
	}
	fclose(conf);	
	return 0;
}

// This always return positive balance so that old-styled ipac supported
float
files_ipac_get_cash(char *login)
{
	return 1;
}

int files_ipac_set_cash(char *login, float cash, timestamp_t timest)
{
	return 0;
};

float files_ipac_get_price(char *rule_name)
{
	return 0;
};

int files_ipac_get_pay_type(char *rule_name)
{
	return 0;
};

timestamp_t files_ipac_get_last_paid(char *service_name)
{
	return 0;
};

int files_ipac_set_last_paid(char *service_name, float cash)
{
	return 0;
};

int files_ipac_close(void)
{
	/* nothing to do. */
	return 0;
}
